import workos
from workos.utils.request import RequestHelper, REQUEST_METHOD_POST
from workos.utils.validation import PASSWORDLESS_MODULE, validate_settings


class Passwordless(object):
    """Offers methods through the WorkOS Passwordless service."""

    @validate_settings(PASSWORDLESS_MODULE)
    def __init__(self):
        pass

    @property
    def request_helper(self):
        if not getattr(self, "_request_helper", None):
            self._request_helper = RequestHelper()
        return self._request_helper

    def create_session(self, session_options):
        """Create a Passwordless Session.

        Args:
            session_options (dict) - An session options object
                session_options[email] (str): The email of the user to authenticate.
                session_options[redirect_uri] (str): Optional parameter to
                    specify the redirect endpoint which will handle the callback
                    from WorkOS. Defaults to the default Redirect URI in the
                    WorkOS dashboard.
                session_options[state] (str): Optional parameter that the redirect
                    URI received from WorkOS will contain. The state parameter
                    can be used to encode arbitrary information to help
                    restore application state between redirects.
                session_options[type] (str): The type of Passwordless Session to
                    create. Currently, the only supported value is 'MagicLink'.

        Returns:
            dict: Passwordless Session
        """

        return self.request_helper.request(
            "passwordless/sessions",
            method=REQUEST_METHOD_POST,
            params=session_options,
            token=workos.api_key,
        )

    def send_session(self, session_id):
        """Send a Passwordless Session via email.

        Args:
            session_id (str): The unique identifier of the Passwordless
                Session to send an email for.

        Returns:
            boolean: Returns True
        """
        self.request_helper.request(
            "passwordless/sessions/{session_id}/send".format(session_id=session_id),
            method=REQUEST_METHOD_POST,
            token=workos.api_key,
        )

        return True



import os

from workos.__about__ import __version__
from workos.client import client

api_key = os.getenv("WORKOS_API_KEY")
project_id = os.getenv("WORKOS_PROJECT_ID")
client_id = os.getenv("WORKOS_CLIENT_ID")
base_api_url = "https://api.workos.com/"



from workos.resources.base import WorkOSBaseResource


class WorkOSProfile(WorkOSBaseResource):
    """Representation of a User Profile as returned by WorkOS through the SSO feature.

    Attributes:
        OBJECT_FIELDS (list): List of fields a WorkOSProfile is comprised of.
    """

    OBJECT_FIELDS = [
        "id",
        "email",
        "first_name",
        "last_name",
        "connection_id",
        "connection_type",
        "idp_id",
        "raw_attributes",
    ]



from workos.resources.base import WorkOSBaseResource
from workos.resources.event_action import WorkOSEventAction


class WorkOSEvent(WorkOSBaseResource):
    """Representation of an Event as returned by WorkOS through the Audit Trail feature.

    Attributes:
        OBJECT_FIELDS (list): List of fields a WorkOSEvent is comprised of.
    """

    OBJECT_FIELDS = [
        "id",
        "group",
        "location",
        "latitude",
        "longitude",
        "type",
        "actor_name",
        "actor_id",
        "target_name",
        "target_id",
        "metadata",
        "occurred_at",
    ]

    @classmethod
    def construct_from_response(cls, response):
        event = super(WorkOSEvent, cls).construct_from_response(response)

        event_action = WorkOSEventAction.construct_from_response(response["action"])
        event.action = event_action

        return event

    def to_dict(self):
        event_dict = super(WorkOSEvent, self).to_dict()

        event_action_dict = self.action.to_dict()
        event_dict["action"] = event_action_dict

        return event_dict



from functools import wraps

import workos
from workos.exceptions import ConfigurationException

AUDIT_TRAIL_MODULE = "AuditTrail"
DIRECTORY_SYNC_MODULE = "DirectorySync"
PASSWORDLESS_MODULE = "Passwordless"
PORTAL_MODULE = "Portal"
SSO_MODULE = "SSO"

REQUIRED_SETTINGS_FOR_MODULE = {
    AUDIT_TRAIL_MODULE: ["api_key",],
    DIRECTORY_SYNC_MODULE: ["api_key",],
    PASSWORDLESS_MODULE: ["api_key",],
    PORTAL_MODULE: ["api_key",],
    SSO_MODULE: ["api_key", "client_id",],
}


def validate_settings(module_name):
    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            missing_settings = []

            # Adding this to accept both client_id and project_id
            # can remove once project_id is deprecated
            if module_name == SSO_MODULE:
                if not getattr(workos, "api_key", None):
                    missing_settings.append("api_key")
                if not getattr(workos, "client_id", None) and not getattr(
                    workos, "project_id", None
                ):
                    missing_settings.append("client_id")
            else:
                for setting in REQUIRED_SETTINGS_FOR_MODULE[module_name]:
                    if not getattr(workos, setting, None):
                        missing_settings.append(setting)

            if missing_settings:
                raise ConfigurationException(
                    "The following settings are missing for {}: {}".format(
                        module_name, ", ".join(missing_settings)
                    )
                )
            return fn(*args, **kwargs)

        return wrapper

    return decorator



class ConfigurationException(Exception):
    pass


# Request related exceptions
class BaseRequestException(Exception):
    def __init__(self, response, message=None, error=None):
        super(BaseRequestException, self).__init__(message)

        self.error = error
        self.extract_and_set_response_related_data(response)

        if message is not None:
            self.message = message

    def extract_and_set_response_related_data(self, response):
        self.response = response

        try:
            response_json = response.json()
            self.message = response_json.get("message")
        except ValueError:
            self.message = None

        headers = response.headers
        self.request_id = headers.get("X-Request-ID")

    def __str__(self):
        message = self.message or "No message"
        exception = "(message=%s" % message

        if self.request_id is not None:
            exception += ", request_id=%s" % self.request_id

        if self.error is not None:
            exception += ", error=%s" % self.error

        return exception + ")"


class AuthorizationException(BaseRequestException):
    pass


class AuthenticationException(BaseRequestException):
    pass


class BadRequestException(BaseRequestException):
    pass


class ServerException(BaseRequestException):
    pass



from enum import Enum


class ConnectionType(Enum):
    ADFSSAML = "ADFSSAML"
    AzureSAML = "AzureSAML"
    GenericOIDC = "GenericOIDC"
    GenericSAML = "GenericSAML"
    GoogleOAuth = "GoogleOAuth"
    GoogleSAML = "GoogleSAML"
    MagicLink = "MagicLink"
    OktaSAML = "OktaSAML"
    OneLoginSAML = "OneLoginSAML"
    PingFederateSAML = "PingFederateSAML"
    PingOneSAML = "PingOneSAML"
    VMwareSAML = "VMwareSAML"



__all__ = [
    "__package_name__",
    "__package_url__",
    "__version__",
    "__author__",
    "__author_email__",
    "__description__",
    "__license__",
]

__package_name__ = "workos"

__package_url__ = "https://github.com/workos-inc/workos-python"

__version__ = "0.8.7"

__author__ = "WorkOS"

__author_email__ = "team@workos.com"

__description__ = "WorkOS Python Client"

__license__ = "MIT"



import json

from requests import Request
from warnings import warn

import workos
from workos.exceptions import ConfigurationException
from workos.resources.sso import WorkOSProfile
from workos.utils.connection_types import ConnectionType
from workos.utils.request import (
    RequestHelper,
    RESPONSE_TYPE_CODE,
    REQUEST_METHOD_DELETE,
    REQUEST_METHOD_GET,
    REQUEST_METHOD_POST,
)
from workos.utils.validation import SSO_MODULE, validate_settings

AUTHORIZATION_PATH = "sso/authorize"
CREATE_CONNECTION_PATH = "connections"
PROMOTE_DRAFT_CONNECTION_PATH = "draft_connections/%s/activate"
TOKEN_PATH = "sso/token"

OAUTH_GRANT_TYPE = "authorization_code"

RESPONSE_LIMIT = 10


class SSO(object):
    """Offers methods to assist in authenticating through the WorkOS SSO service."""

    @validate_settings(SSO_MODULE)
    def __init__(self):
        pass

    @property
    def request_helper(self):
        if not getattr(self, "_request_helper", None):
            self._request_helper = RequestHelper()
        return self._request_helper

    def get_authorization_url(
        self, domain=None, redirect_uri=None, state=None, provider=None, connection=None
    ):
        """Generate an OAuth 2.0 authorization URL.

        The URL generated will redirect a User to the Identity Provider configured through
        WorkOS.

        Kwargs:
            domain (str) - The domain a user is associated with, as configured on WorkOS
            redirect_uri (str) - A valid redirect URI, as specified on WorkOS
            state (str) - An encoded string passed to WorkOS that'd be preserved through the authentication workflow, passed
            back as a query parameter
            provider (ConnectionType) - Authentication service provider descriptor
            connection (string) - Unique identifier for a WorkOS Connection

        Returns:
            str: URL to redirect a User to to begin the OAuth workflow with WorkOS
        """
        params = {
            "client_id": workos.client_id,
            "redirect_uri": redirect_uri,
            "response_type": RESPONSE_TYPE_CODE,
        }

        if domain is None and provider is None and connection is None:
            raise ValueError(
                "Incomplete arguments. Need to specify either a 'connection', 'domain' or 'provider'"
            )
        if provider is not None:
            if not isinstance(provider, ConnectionType):
                raise ValueError("'provider' must be of type ConnectionType")
            params["provider"] = str(provider.value)
        if domain is not None:
            params["domain"] = domain
        if connection is not None:
            params["connection"] = connection

        if state is not None:
            params["state"] = state

        if workos.project_id is not None:
            params["client_id"] = workos.project_id
            warn(
                "'project_id' is deprecated. Use 'client_id' instead.",
                DeprecationWarning,
            )

        prepared_request = Request(
            "GET",
            self.request_helper.generate_api_url(AUTHORIZATION_PATH),
            params=params,
        ).prepare()

        return prepared_request.url

    def get_profile(self, code):
        """Get the profile of an authenticated User

        Once authenticated, using the code returned having followed the authorization URL,
        get the WorkOS profile of the User.

        Args:
            code (str): Code returned by WorkOS on completion of OAuth 2.0 workflow

        Returns:
            WorkOSProfile: WorkOSProfile object representing the User
        """
        params = {
            "client_id": workos.client_id,
            "client_secret": workos.api_key,
            "code": code,
            "grant_type": OAUTH_GRANT_TYPE,
        }

        if workos.project_id is not None:
            params["client_id"] = workos.project_id
            warn(
                "'project_id' is deprecated. Use 'client_id' instead.",
                DeprecationWarning,
            )

        response = self.request_helper.request(
            TOKEN_PATH, method=REQUEST_METHOD_POST, params=params
        )

        return WorkOSProfile.construct_from_response(response["profile"])

    def promote_draft_connection(self, token):
        """Promote a Draft Connection

        Promotes a Draft Connection created through the WorkOS.js embed. A Draft Connection that has
        been promoted will enable Enterprise users of the domain to begin signing in via SSO.

        Args:
            token (str): The token supplied via the response when a draft connection is created via
            the WorkOS.js embed

        Returns:
            bool: True if a Draft Connection has been successfully promoted
        """
        warn(
            "'promote_draft_connection' is deprecated. Use 'create_connection' instead.",
            DeprecationWarning,
        )
        self.request_helper.request(
            PROMOTE_DRAFT_CONNECTION_PATH % token,
            method=REQUEST_METHOD_POST,
            token=workos.api_key,
        )

        return True

    def create_connection(self, source):
        """Activates a Draft Connection created through the WorkOS.js widget.

        Args:
            source (str): Draft Connection identifier.

        Returns:
            dict: Created Connection response from WorkOS.
        """
        params = {"source": source}
        return self.request_helper.request(
            CREATE_CONNECTION_PATH,
            method=REQUEST_METHOD_POST,
            params=params,
            token=workos.api_key,
        )

    def get_connection(self, connection):
        """Gets details for a single Connection

        Args:
            connection (str): Connection unique identifier

        Returns:
            dict: Connection response from WorkOS.
        """
        return self.request_helper.request(
            "connections/{connection}".format(connection=connection),
            method=REQUEST_METHOD_GET,
            token=workos.api_key,
        )

    def list_connections(
        self,
        connection_type=None,
        domain=None,
        organization_id=None,
        limit=RESPONSE_LIMIT,
        before=None,
        after=None,
    ):
        """Gets details for existing Connections.

        Args:
            connection_type (ConnectionType): Authentication service provider descriptor. (Optional)
            domain (str): Domain of a Connection. (Optional)
            limit (int): Maximum number of records to return. (Optional)
            before (str): Pagination cursor to receive records before a provided Connection ID. (Optional)
            after (str): Pagination cursor to receive records after a provided Connection ID. (Optional)

        Returns:
            dict: Connections response from WorkOS.
        """
        params = {
            "connection_type": connection_type,
            "domain": domain,
            "organization_id": organization_id,
            "limit": limit,
            "before": before,
            "after": after,
        }
        return self.request_helper.request(
            "connections",
            method=REQUEST_METHOD_GET,
            params=params,
            token=workos.api_key,
        )

    def delete_connection(self, connection):
        """Deletes a single Connection

        Args:
            connection (str): Connection unique identifier
        """
        return self.request_helper.request(
            "connections/{connection}".format(connection=connection),
            method=REQUEST_METHOD_DELETE,
            token=workos.api_key,
        )



import workos
from workos.exceptions import ConfigurationException
from workos.resources.event import WorkOSEvent
from workos.utils.request import RequestHelper, REQUEST_METHOD_GET, REQUEST_METHOD_POST
from workos.utils.validation import AUDIT_TRAIL_MODULE, validate_settings

EVENTS_PATH = "events"
METADATA_LIMIT = 50
DEFAULT_EVENT_LIMIT = 10


class AuditTrail(object):
    """Offers methods through the WorkOS Audit Trail service."""

    @validate_settings(AUDIT_TRAIL_MODULE)
    def __init__(self):
        pass

    @property
    def request_helper(self):
        if not getattr(self, "_request_helper", None):
            self._request_helper = RequestHelper()
        return self._request_helper

    def create_event(self, event, idempotency_key=None):
        """Create an Audit Trail event.

        Args:
            event (dict) - An event object
                event[action] (str) - Specific activity performed by the actor.
                event[action_type] (str) - Corresponding CRUD category of the
                    event. Can be one of C, R, U, or D.
                event[actor_name] (str) - Display name of the entity performing the action
                event[actor_id] (str) - Unique identifier of the entity performing the action
                event[group] (str) - A single organization containing related .
                    members. This will normally be the customer of a vendor's application
                event[location] (str) - Identifier for where the event
                    originated. This will be an IP address (IPv4 or IPv6),
                    hostname, or device ID.
                event[occurred_at] (str) - ISO-8601 datetime at which the event
                    happened, with millisecond precision.
                event[metadata] (str) - Arbitrary key-value data containing
                    information associated with the event. Note: There is a limit of 50
                    keys. Key names can be up to 40 characters long, and values can be up
                    to 500 characters long.
                event[target_id] (str) - Unique identifier of the object or
                    resource being acted upon.
                event[target_name] (str) - Display name of the object or
                    resource that is being acted upon.
            idempotency_key (str) - An idempotency key

        Returns:
            boolean: Returns True
        """
        if len(event.get("metadata", {})) > METADATA_LIMIT:
            raise ValueError(
                "Number of metadata keys exceeds {}.".format(METADATA_LIMIT)
            )

        headers = {
            "idempotency-key": idempotency_key,
        }

        self.request_helper.request(
            EVENTS_PATH,
            method=REQUEST_METHOD_POST,
            params=event,
            headers=headers,
            token=workos.api_key,
        )

        return True

    def get_events(
        self,
        before=None,
        after=None,
        limit=DEFAULT_EVENT_LIMIT,
        group=None,
        action=None,
        action_type=None,
        actor_name=None,
        actor_id=None,
        target_name=None,
        target_id=None,
        occurred_at=None,
        occurred_at_gt=None,
        occurred_at_gte=None,
        occurred_at_lt=None,
        occurred_at_lte=None,
        search=None,
    ):
        """Filter for Audit Trail Events.

        Kwargs:
            before (str) - Event ID to look before
            after (str) - Event ID to look after
            limit (int) - Number of Events to return
            group (str|list) - Group or groups to filter for
            action (str|list) - Action or actions to filter for
            action_type (str|list) - Action type or types to filter for
            actor_name (str|list) - Actor name or name to filter for
            actor_id (str|list) - Actor ID or IDs to filter for
            target_name (str|list) - Target name or names to filter for
            target_id (str|list) - Target ID or IDs to filter for
            occurred_at (str) - ISO-8601 datetime of when an event occurred
            occurred_at_gt (str) - ISO-8601 datetime of when an event occurred after
            occurred_at_gte (str) - ISO-8601 datetime of when an event occurred at or after
            occurred_at_lt (str) - ISO-8601 datetime of when an event occurred before
            occurred_at_lte (str) - ISO-8601 datetime of when an event occured at or before
            search (str) - Keyword search

        Returns:
            tuple
                list - List of WorkOSEvent objects
                string - Event ID to use as before cursor
                string - Event ID to use as after cursor
        """
        params = {
            "before": before,
            "after": after,
            "limit": limit,
        }

        if group is not None:
            params["group[]"] = group

        if action is not None:
            params["action[]"] = action

        if action_type is not None:
            params["action_type[]"] = action_type

        if actor_name is not None:
            params["actor_name[]"] = actor_name

        if actor_id is not None:
            params["actor_id[]"] = actor_id

        if target_name is not None:
            params["target_name[]"] = target_name

        if target_id is not None:
            params["target_id[]"] = target_id

        if occurred_at is not None:
            params["occurred_at"] = occurred_at
        else:
            if occurred_at_gte is not None:
                params["occurred_at_gte"] = occurred_at_gte
            elif occurred_at_gt is not None:
                params["occurred_at_gt"] = occurred_at_gt

            if occurred_at_lte is not None:
                params["occurred_at_lte"] = occurred_at_lte
            elif occurred_at_lt is not None:
                params["occurred_at_lt"] = occurred_at_lt

        if search is not None:
            params["search"] = search

        response = self.request_helper.request(
            EVENTS_PATH, method=REQUEST_METHOD_GET, params=params, token=workos.api_key,
        )

        events = [
            WorkOSEvent.construct_from_response(data) for data in response["data"]
        ]
        before = response["listMetadata"]["before"]
        after = response["listMetadata"]["after"]

        return (events, before, after)



import requests

import workos
from workos.exceptions import (
    AuthorizationException,
    AuthenticationException,
    BadRequestException,
    ServerException,
)

BASE_HEADERS = {
    "User-Agent": "WorkOS Python/{}".format(workos.__version__),
}

RESPONSE_TYPE_CODE = "code"

REQUEST_METHOD_DELETE = "delete"
REQUEST_METHOD_GET = "get"
REQUEST_METHOD_POST = "post"


class RequestHelper(object):
    def __init__(self):
        self.set_base_api_url(workos.base_api_url)

    def set_base_api_url(self, base_api_url):
        """Creates an accessible template for constructing the URL for an API request.

        Args:
            base_api_url (str): Base URL for api requests (Should end with a /)
        """
        self.base_api_url = "{}{{}}".format(base_api_url)

    def generate_api_url(self, path):
        return self.base_api_url.format(path)

    def request(
        self, path, method=REQUEST_METHOD_GET, params=None, headers=None, token=None,
    ):
        """Executes a request against the WorkOS API.

        Args:
            path (str): Path for the api request that'd be appended to the base API URL

        Kwargs:
            method (str): One of the supported methods as defined by the REQUEST_METHOD_X constants
            params (dict): Query params to be added to the request
            token (str): Bearer token

        Returns:
            dict: Response from WorkOS
        """
        if headers is None:
            headers = {}

        if token:
            headers["Authorization"] = "Bearer {}".format(token)

        headers.update(BASE_HEADERS)
        url = self.generate_api_url(path)

        request_fn = getattr(requests, method)
        if method == REQUEST_METHOD_GET:
            response = request_fn(url, headers=headers, params=params)
        else:
            response = request_fn(url, headers=headers, json=params)

        response_json = None
        try:
            response_json = response.json()
        except ValueError:
            raise ServerException(response)

        status_code = response.status_code
        if status_code >= 400 and status_code < 500:
            if status_code == 401:
                raise AuthenticationException(response)
            elif status_code == 403:
                raise AuthorizationException(response)
            error = response_json.get("error")
            raise BadRequestException(response, error=error)
        elif status_code >= 500 and status_code < 600:
            raise ServerException(response)

        return response_json



class WorkOSBaseResource(object):
    """Representation of a WorkOS Resource as returned through the API.

    Attributes:
        OBJECT_FIELDS (list): List of fields a Resource is comprised of.
    """

    OBJECT_FIELDS = []

    @classmethod
    def construct_from_response(cls, response):
        """Returns an instance of WorkOSBaseResource.

        Args:
            response (dict): Resource data from a WorkOS API response

        Returns:
            WorkOSBaseResource: Instance of a WorkOSBaseResource with OBJECT_FIELDS fields set
        """
        obj = cls()
        for field in cls.OBJECT_FIELDS:
            setattr(obj, field, response[field])

        return obj

    def to_dict(self):
        """Returns a dict representation of the WorkOSBaseResource.

        Returns:
            dict: A dict representation of the WorkOSBaseResource
        """
        obj_dict = {}
        for field in self.OBJECT_FIELDS:
            obj_dict[field] = getattr(self, field, None)

        return obj_dict



import workos
from workos.utils.request import RequestHelper, REQUEST_METHOD_GET
from workos.utils.validation import DIRECTORY_SYNC_MODULE, validate_settings

RESPONSE_LIMIT = 10


class DirectorySync(object):
    """Offers methods through the WorkOS Directory Sync service."""

    @validate_settings(DIRECTORY_SYNC_MODULE)
    def __init__(self):
        pass

    @property
    def request_helper(self):
        if not getattr(self, "_request_helper", None):
            self._request_helper = RequestHelper()
        return self._request_helper

    def list_users(
        self, directory=None, group=None, limit=RESPONSE_LIMIT, before=None, after=None
    ):
        """Gets a list of provisioned Users for a Directory.

        Note, either 'directory' or 'group' must be provided.

        Args:
            directory (str): Directory unique identifier.
            group (str): Directory Group unique identifier.
            limit (int): Maximum number of records to return.
            before (str): Pagination cursor to receive records before a provided Directory ID.
            after (str): Pagination cursor to receive records after a provided Directory ID.

        Returns:
            dict: Directory Users response from WorkOS.
        """
        params = {"limit": limit, "before": before, "after": after}
        if group is not None:
            params["group"] = group
        if directory is not None:
            params["directory"] = directory
        return self.request_helper.request(
            "directory_users",
            method=REQUEST_METHOD_GET,
            params=params,
            token=workos.api_key,
        )

    def list_groups(
        self, directory=None, user=None, limit=RESPONSE_LIMIT, before=None, after=None
    ):
        """Gets a list of provisioned Groups for a Directory .

        Note, either 'directory' or 'user' must be provided.

        Args:
            directory (str): Directory unique identifier.
            user (str): Directory User unique identifier.
            limit (int): Maximum number of records to return.
            before (str): Pagination cursor to receive records before a provided Directory ID.
            after (str): Pagination cursor to receive records after a provided Directory ID.

        Returns:
            dict: Directory Groups response from WorkOS.
        """
        params = {"limit": limit, "before": before, "after": after}
        if user is not None:
            params["user"] = user
        if directory is not None:
            params["directory"] = directory
        return self.request_helper.request(
            "directory_groups",
            method=REQUEST_METHOD_GET,
            params=params,
            token=workos.api_key,
        )

    def get_user(self, user):
        """Gets details for a single provisioned Directory User.

        Args:
            user (str): Directory User unique identifier.

        Returns:
            dict: Directory User response from WorkOS.
        """
        return self.request_helper.request(
            "directory_users/{user}".format(user=user),
            method=REQUEST_METHOD_GET,
            token=workos.api_key,
        )

    def get_group(self, group):
        """Gets details for a single provisioned Directory Group.

        Args:
            group (str): Directory Group unique identifier.

        Returns:
            dict: Directory Group response from WorkOS.
        """
        return self.request_helper.request(
            "directory_groups/{group}".format(group=group),
            method=REQUEST_METHOD_GET,
            token=workos.api_key,
        )

    def list_directories(
        self, domain=None, search=None, limit=RESPONSE_LIMIT, before=None, after=None
    ):
        """Gets details for existing Directories.

        Args:
            domain (str): Domain of a Directory. (Optional)
            search (str): Searchable text for a Directory. (Optional)
            limit (int): Maximum number of records to return. (Optional)
            before (str): Pagination cursor to receive records before a provided Directory ID. (Optional)
            after (str): Pagination cursor to receive records after a provided Directory ID. (Optional)

        Returns:
            dict: Directories response from WorkOS.
        """
        params = {
            "domain": domain,
            "search": search,
            "limit": limit,
            "before": before,
            "after": after,
        }
        return self.request_helper.request(
            "directories",
            method=REQUEST_METHOD_GET,
            params=params,
            token=workos.api_key,
        )



from workos.audit_trail import AuditTrail
from workos.directory_sync import DirectorySync
from workos.passwordless import Passwordless
from workos.portal import Portal
from workos.sso import SSO


class Client(object):
    """Client for a convenient way to access the WorkOS feature set."""

    @property
    def sso(self):
        if not getattr(self, "_sso", None):
            self._sso = SSO()
        return self._sso

    @property
    def audit_trail(self):
        if not getattr(self, "_audit_trail", None):
            self._audit_trail = AuditTrail()
        return self._audit_trail

    @property
    def directory_sync(self):
        if not getattr(self, "_directory_sync", None):
            self._directory_sync = DirectorySync()
        return self._directory_sync

    @property
    def passwordless(self):
        if not getattr(self, "_passwordless", None):
            self._passwordless = Passwordless()
        return self._passwordless

    @property
    def portal(self):
        if not getattr(self, "_portal", None):
            self._portal = Portal()
        return self._portal


client = Client()



from workos.resources.base import WorkOSBaseResource


class WorkOSEventAction(WorkOSBaseResource):
    """Representation of an Event Action as returned by WorkOS through the Audit Trail feature.

    Attributes:
        OBJECT_FIELDS (list): List of fields a WorkOSEventAction is comprised of.
    """

    OBJECT_FIELDS = ["id", "name", "environment_id"]



import os
import sys
from setuptools import setup, find_packages

base_dir = os.path.dirname(__file__)

with open("README.md", "r") as f:
    long_description = f.read()

about = {}
with open(os.path.join(base_dir, "workos", "__about__.py")) as f:
    exec(f.read(), about)

dev_requirements = [
    "flake8",
    "pytest==4.6.9",
    "pytest-cov==2.8.1",
    "six==1.13.0",
]
if sys.version_info.major == 3:
    dev_requirements.extend(
        ["black==19.10b0", "twine==3.1.1",]
    )

setup(
    name=about["__package_name__"],
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__package_url__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests*",]),
    zip_safe=False,
    license=about["__license__"],
    install_requires=["requests>=2.22.0"],
    extras_require={"dev": dev_requirements, ":python_version<'3.4'": ["enum34"],},
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import workos
from workos.utils.request import RequestHelper, REQUEST_METHOD_GET, REQUEST_METHOD_POST
from workos.utils.validation import PORTAL_MODULE, validate_settings

ORGANIZATIONS_PATH = "organizations"
PORTAL_GENERATE_PATH = "portal/generate_link"
RESPONSE_LIMIT = 10


class Portal(object):
    @validate_settings(PORTAL_MODULE)
    def __init__(self):
        pass

    @property
    def request_helper(self):
        if not getattr(self, "_request_helper", None):
            self._request_helper = RequestHelper()
        return self._request_helper

    def create_organization(self, organization):
        """Create an organization

        Args:
            organization (dict) - An organization object
                organization[domains] (list) - List of domains that belong to the organization
                organization[name] (str) - A unique, descriptive name for the organization

        Returns:
            dict: Created Organization response from WorkOS.
        """
        return self.request_helper.request(
            ORGANIZATIONS_PATH,
            method=REQUEST_METHOD_POST,
            params=organization,
            token=workos.api_key,
        )

    def generate_link(self, intent, organization, return_url=None):
        """Generate a link to grant access to an organization's Admin Portal

        Args:
            intent (str): The access scope for the generated Admin Portal link. Valid values are: ["sso", "dsync"]
            organization (string): The ID of the organization the Admin Portal link will be generated for

        Kwargs:
            return_url (str): The URL that the end user will be redirected to upon exiting the generated Admin Portal. If none is provided, the default redirect link set in your WorkOS Dashboard will be used. (Optional)

        Returns:
            str:  URL to redirect a User to to access an Admin Portal session
        """
        params = {
            "intent": intent,
            "organization": organization,
            "return_url": return_url,
        }
        return self.request_helper.request(
            PORTAL_GENERATE_PATH,
            method=REQUEST_METHOD_POST,
            params=params,
            token=workos.api_key,
        )

    def list_organizations(
        self, domains=None, limit=RESPONSE_LIMIT, before=None, after=None
    ):
        """Retrieve a list of organizations that have connections configured within your WorkOS dashboard.

        Kwargs:
            domains (list): Filter organizations to only return those that are associated with the provided domains. (Optional)
            limit (int): Maximum number of records to return. (Optional)
            before (str): Pagination cursor to receive records before a provided Organization ID. (Optional)
            after (str): Pagination cursor to receive records after a provided Organization ID. (Optional)

        Returns:
            dict: Organizations response from WorkOS.
        """
        params = {
            "domains": domains,
            "limit": limit,
            "before": before,
            "after": after,
        }
        return self.request_helper.request(
            ORGANIZATIONS_PATH,
            method=REQUEST_METHOD_GET,
            params=params,
            token=workos.api_key,
        )
